﻿/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "lpc177x_8x_pinsel.h"
#include "lpc177x_8x_timer.h"
#include "lpc177x_8x_i2c.h"
#include "lpc177x_8x_ssp.h"
#include "system_lpc177x_8x.h"

#include "SEC_LPC1788OEM_BB.h"
#include "pca9532.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/
#define SSP_CLOCK 500000

#define I2CDEV I2C_0
#define LM75A_I2C_ADDR 0x49 //(0x49 << 1)
#define LM75A_CMD_TEMP 0x00


/******************************************************************************
 * Local variables
 *****************************************************************************/
static void (*timerCallb)(void) = NULL;
static uint32_t msTicks = 0;
//static uint8_t ms100 = 0;


/******************************************************************************
 * Local functions
 *****************************************************************************/
static int I2CRead(uint8_t addr, uint8_t* buf, uint32_t len)
{
	I2C_M_SETUP_Type rxsetup;

	rxsetup.sl_addr7bit = addr;
	rxsetup.tx_data = NULL;	// Get address to read at writing address
	rxsetup.tx_length = 0;
	rxsetup.rx_data = buf;
	rxsetup.rx_length = len;
	rxsetup.retransmissions_max = 3;

	if (I2C_MasterTransferData(I2CDEV, &rxsetup, I2C_TRANSFER_POLLING) == SUCCESS){
		return (0);
	} else {
		return (-1);
	}
}

static int I2CWrite(uint8_t addr, uint8_t* buf, uint32_t len)
{
	I2C_M_SETUP_Type txsetup;

	txsetup.sl_addr7bit = addr;
	txsetup.tx_data = buf;
	txsetup.tx_length = len;
	txsetup.rx_data = NULL;
	txsetup.rx_length = 0;
	txsetup.retransmissions_max = 3;

	if (I2C_MasterTransferData(I2CDEV, &txsetup, I2C_TRANSFER_POLLING) == SUCCESS){
		return (0);
	} else {
		return (-1);
	}
}

static int32_t lm75a_readTemp(void)
{
    uint8_t temp[2];
    int16_t t = 0;

    I2CWrite(LM75A_I2C_ADDR, LM75A_CMD_TEMP, 1);
    I2CRead(LM75A_I2C_ADDR, temp, 2);

    /* 11 MSB bits used. Celcius is calculated as Temp data * 1/8 */
    t = ((temp[0] << 8) | (temp[1]));

    return ((t * 100) >> 8);
}

/******************************************************************************
 *
 * Description:
 *   Initialize I2C0
 *
 *****************************************************************************/
static void i2c0_init(void)
{

	/* Initialize I2C0 pin connect */
	PINSEL_ConfigPin(0,27,1);
	PINSEL_ConfigPin(0,28,1);

	// Initialize I2C0 peripheral
	I2C_Init(I2CDEV, 100000);

	/* Enable I2C0 operation */
	I2C_Cmd(I2CDEV, I2C_MASTER_MODE, ENABLE);
}

/******************************************************************************
 * Public functions
 *****************************************************************************/
uint32_t bsp_getMsTicks(void)
{
  return msTicks;
}

void SysTick_Handler(void)
{
	msTicks++;
	if ((timerCallb != NULL) && ((msTicks % 1000) == 0))
	{
//		msTicks = 0;
		timerCallb();
	}
}

void bsp_spiInit(void)
{
	SSP_CFG_Type SSP_ConfigStruct;

	SEC14_INIT_SPI;

	// initialize SSP configuration structure to default
	SSP_ConfigStructInit(&SSP_ConfigStruct);

	// set clock rate
	SSP_ConfigStruct.ClockRate = SSP_CLOCK;

	// set number of databits
	SSP_ConfigStruct.Databit = SSP_DATABIT_8;

	SSP_ConfigStruct.CPHA = SSP_CPHA_FIRST;
	SSP_ConfigStruct.CPOL = SSP_CPOL_HI;

	// Initialize SSP peripheral with parameter given in structure above
	SSP_Init(SSP_PORT, &SSP_ConfigStruct);

	// Enable SSP peripheral
	SSP_Cmd(SSP_PORT, ENABLE);

	//pulldown on MOSI-pin (P0.18)
	PINSEL_SetPinMode(0, 18, PINSEL_BASICMODE_PULLDOWN);
}

/******************************************************************************
 *
 * Description:
 *   BSP initialize function. Must be called before any other BSP related
 *   functions.
 *
 *****************************************************************************/
void bsp_init(void)
{
	TIM_TIMERCFG_Type timerCfg;

	/* Initialize I2C and LM75 */
	i2c0_init();
	lm75a_readTemp();

	TIM_ConfigStructInit(TIM_TIMER_MODE, &timerCfg);
	TIM_Init(LPC_TIM0, TIM_TIMER_MODE, &timerCfg);

	bsp_spiInit();
}

void bsp_spiWrite(uint8_t* data, uint16_t len)
{
  SSP_DATA_SETUP_Type sspCfg;

  sspCfg.tx_data = data;
  sspCfg.rx_data = NULL;
  sspCfg.length  = len;
  SSP_ReadWrite (SSP_PORT, &sspCfg, SSP_TRANSFER_POLLING);
}


void bsp_spiRead(uint8_t* data, uint16_t len)
{
  SSP_DATA_SETUP_Type sspCfg;

  sspCfg.rx_data = data;
  sspCfg.tx_data = NULL;
  sspCfg.length  = len;
  SSP_ReadWrite (SSP_PORT, &sspCfg, SSP_TRANSFER_POLLING);
}


/******************************************************************************
 *
 * Description:
 *   Start a timer.
 *
 * Params:
 *   [in] callb - callback function called at given frequency
 *
 *****************************************************************************/
void bsp_startTimer(void (*callb)(void))
{
  timerCallb = callb;
  SysTick_Config(SystemCoreClock/1000);
}

/******************************************************************************
 *
 * Description:
 *   Delay specified number of milliseconds
 *
 * Params:
 *   [in] ms - number of milliseconds to delay
 *
 *****************************************************************************/
void bsp_delayMs(uint32_t ms)
{
  TIM_Waitms(ms);
}

/******************************************************************************
 *
 * Description:
 *   Delay specified number of microseconds
 *
 * Params:
 *   [in] us - number of microseconds to delay
 *
 *****************************************************************************/
void bsp_delayUs(uint32_t us)
{
  TIM_Waitus(us);
}

/******************************************************************************
 *
 * Description:
 *   Read temperature
 *
 * Returns:
 *   (int16_t) temperature in Celsius
 *
 *****************************************************************************/
int16_t bsp_getTemp(void)
{
	lm75a_readTemp();
	TIM_Waitms(5);
	return (int16_t)(lm75a_readTemp()/100);
}

/******************************************************************************
 *
 * Description:
 *   Get button state
 *
 * Returns:
 *   [out]
 *
 *****************************************************************************/
uint8_t bsp_getKey(void)
{
//	if ((GPIO_ReadValue(2) & (1<<11)) == 0)
//		return KEY_PRESS;
//	return KEY_NO_PRESS;
  uint16_t state = 0;

  //
  // SW2 button on the base board is used
  //

  state = pca9532_getLedState(FALSE);
  if (state & KEY1) {
    return KEY_PRESS;
  }
  else {
    return KEY_NO_PRESS;
  }
}

void bsp_pinDir(uint8_t pin, uint8_t dir)
{
  switch(pin) {

  case 6:
    // p2.14
    GPIO_SetDir(2, (1<<14), dir);

    break;

  case 7:
    // p0.11
    GPIO_SetDir(0, (1<<11), dir);

    break;
  case 8:
    // p0.10
    GPIO_SetDir(0, (1<<10), dir);

    break;
  case 11:
    // p1.18
    GPIO_SetDir(1, (1<<18), dir);

    break;
  case 12:
    // p2.21
    GPIO_SetDir(2, (1<<21), dir);

    break;
  case 13:
    // p0.23
    GPIO_SetDir(0, (1<<23), dir);

    break;
  case 14:
    // p0.24
    GPIO_SetDir(0, (1<<24), dir);

    break;
  default:
    // ignore other pin values
    break;
  }
}

void bsp_pinSet(uint8_t pin)
{
  switch(pin) {


  case 6:
    // p2.14
    GPIO_SetValue(2, (1<<14));

    break;

  case 7:
    // p0.11
    GPIO_SetValue(0, (1<<11));

    break;
  case 8:
    // p0.10
    GPIO_SetValue(0, (1<<10));

    break;
  case 11:
    // p1.18
    GPIO_SetValue(1, (1<<18));

    break;
  case 12:
    // p2.21
    GPIO_SetValue(2, (1<<21));

    break;
  case 13:
    // p0.23
    GPIO_SetValue(0, (1<<23));

    break;
  case 14:
    // p0.24
    GPIO_SetValue(0, (1<<24));

    break;
  default:
    // ignore other pin values
    break;
  }
}

void bsp_pinClr(uint8_t pin)
{
  switch(pin) {


  case 6:
    // p2.14
    GPIO_ClearValue(2, (1<<14));

    break;

  case 7:
    // p0.11
    GPIO_ClearValue(0, (1<<11));

    break;
  case 8:
    // p0.10
    GPIO_ClearValue(0, (1<<10));

    break;
  case 11:
    // p1.18
    GPIO_ClearValue(1, (1<<18));

    break;
  case 12:
    // p2.21
    GPIO_ClearValue(2, (1<<21));

    break;
  case 13:
    // p0.23
    GPIO_ClearValue(0, (1<<23));

    break;
  case 14:
    // p0.24
    GPIO_ClearValue(0, (1<<24));

    break;
  default:
    // ignore other pin values
    break;
  }
}

uint8_t bsp_pinState(uint8_t pin)
{
  uint8_t result = 0;

  switch(pin) {

  case 6:
    // p2.14
    result = ((GPIO_ReadValue(2) & (1<<14)) != 0);

    break;

  case 7:
    // p0.11
    result = ((GPIO_ReadValue(0) & (1<<11)) != 0);

    break;
  case 8:
    // p0.10
    result = ((GPIO_ReadValue(0) & (1<<10)) != 0);

    break;
  case 11:
    // p1.18
    result = ((GPIO_ReadValue(1) & (1<<18)) != 0);

    break;
  case 12:
    // p2.21
    result = ((GPIO_ReadValue(2) & (1<<21)) != 0);

    break;
  case 13:
    // p0.23
    result = ((GPIO_ReadValue(0) & (1<<23)) != 0);

    break;
  case 14:
    // p0.24
    result = ((GPIO_ReadValue(0) & (1<<24)) != 0);

    break;
  default:
    // ignore other pin values
    break;
  }

  return result;
}

void bsp_setLed(uint8_t on)
{
  //
  // LED22 on the base board is used
  //

  if (on) {
    pca9532_setLeds(LED1, 0);
  }
  else {
    pca9532_setLeds(0, LED1);
  }
}


/******************************************************************************
 *
 * Description:
 *   Turn the specified LED on or off
 *
 * Params:
 *   [in] on - 1 if the LED should be turned on; otherwise 0.
 *
 *****************************************************************************/
void bsp_setRgbLed(rgbled_t led, uint8_t on)
{
	uint16_t mask = 0;
	switch (led) {
	case LED6_RED:
		mask = LED1;
		break;

	case LED6_GREEN:
		mask = LED5;
		break;

	case LED6_BLUE:
		mask = LED3;
		break;

	case LED7_RED:
		mask = LED2;
		break;

	case LED7_GREEN:
		mask = LED6;
		break;

	case LED7_BLUE:
		mask = LED4;
		break;
	}

	if (on) {
		pca9532_setLeds(mask, 0);
	}
	else {
		pca9532_setLeds(0, mask);
	}
}

