/**********************************************************************
 *
 * Copyright(C) 2013, Embedded Artists AB
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/


/******************************************************************************
 * Includes
 *****************************************************************************/

#include "flash.h"


/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/


#ifndef MIN
#define MIN(x, y) ((x) < (y) ? (x) : (y))
#endif


#define FLASH_CMD_RDID      0x9F        /* read device ID */
#define FLASH_CMD_RDSR1     0x05        /* read status 1 register */
#define FLASH_CMD_RDSR2     0x35        /* read status 2 register */
#define FLASH_CMD_RES       0xAB        /* release from deep power down */
#define FLASH_CMD_WREN      0x06        /* write enable */
#define FLASH_CMD_FAST_READ 0x0B        /* read data (extra command setup time allows higher SPI clock) */
#define FLASH_CMD_PP        0x02        /* page program */
#define FLASH_CMD_SE        0x20        /* sector erase */

/* status register defines */
#define STATUS_1_BUSY (1 << 0)
#define STATUS_1_WEL  (1 << 1)
#define STATUS_1_BP0  (1 << 2)
#define STATUS_1_BP1  (1 << 3)
#define STATUS_1_BP2  (1 << 4)
#define STATUS_1_TB   (1 << 5)
#define STATUS_1_SEC  (1 << 6)
#define STATUS_1_SRP0 (1 << 7)

struct _flash_info
{
  char* name;
  uint32_t jedec_id;
  uint32_t nr_pages;
  uint16_t pagesize;
  uint16_t sectorSize;
  uint8_t flags;
};

#define FLASH_CS_LOW  (bsp_pinClr(6))
#define FLASH_CS_HIGH (bsp_pinSet(6))


/******************************************************************************
 * External global variables
 *****************************************************************************/

/******************************************************************************
 * Local variables
 *****************************************************************************/


static struct _flash_info flash_devices[] = {
    {"S25FL004K", 0xEF4013, 2048, 256, 4096, 0},
    {"S25FL008K", 0xEF4014, 4096, 256, 4096, 0},
    {"W25Q32B", 0xEF4016, 131072, 256, 4096, 0},
};

static uint16_t pageSize = 0;
static uint16_t sectorSize = 0;
static uint32_t flashTotalSize = 0;

/******************************************************************************
 * Local Functions
 *****************************************************************************/

static void exitDeepPowerDown(void)
{
  uint8_t cmd = FLASH_CMD_RES;

  FLASH_CS_LOW;
  bsp_spiWrite(&cmd, 1);
  FLASH_CS_HIGH;
}

static void writeEnable(void)
{
  uint8_t cmd = FLASH_CMD_WREN;

  FLASH_CS_LOW;
  bsp_spiWrite(&cmd, 1);
  FLASH_CS_HIGH;
}

static void readDeviceId(uint8_t *deviceId)
{
  uint8_t cmd = FLASH_CMD_RDID;

  FLASH_CS_LOW;
  bsp_spiWrite(&cmd, 1);
  bsp_spiRead((uint8_t *)deviceId, 4);
  FLASH_CS_HIGH;
}

static uint8_t readStatus1(void)
{
  uint8_t cmd = FLASH_CMD_RDSR1;
  uint8_t status = 0;

  FLASH_CS_LOW;
  bsp_spiWrite(&cmd, 1);
  bsp_spiRead((uint8_t *)&status, 1 );
  FLASH_CS_HIGH;

  return status;
}

static void pollIsBusy(void)
{
  uint8_t status = 0;
  int i = 0;

  do
  {
    for (i = 0; i < 0x2000; i++);

    status = readStatus1();
  }
  while ((status & STATUS_1_BUSY) != 0);
}



/******************************************************************************
 * Public Functions
 *****************************************************************************/

/******************************************************************************
 *
 * Description:
 *    Initialize the flash Driver
 *
 *****************************************************************************/
uint32_t flash_init (void)
{
  uint8_t deviceId[4];
  uint8_t status = 0;
  uint32_t id = 0;
  int i = 0;


  exitDeepPowerDown();
  readDeviceId(deviceId);



  id = (deviceId[0] << 16 | deviceId[1] << 8 | deviceId[2]);

  for (i = 0; i < sizeof(flash_devices) / sizeof(struct _flash_info); i++) {
    if (flash_devices[i].jedec_id == id)
    {
      pageSize   = flash_devices[i].pagesize;
      sectorSize = flash_devices[i].sectorSize;
      flashTotalSize = flash_devices[i].pagesize * flash_devices[i].nr_pages;

      do {
        status = readStatus1();
      } while ((status & STATUS_1_BUSY) != 0);

      return TRUE;
    }
  }


  return FALSE;
}

/******************************************************************************
 *
 * Description:
 *    Read data from flash
 *
 * Params:
 *   [in] buf - data buffer
 *   [in] offset - offset into the flash
 *   [in] len - number of bytes to read
 *
 * Returns:
 *   number of read bytes
 *
 *****************************************************************************/
uint32_t flash_read(uint8_t* buf, uint32_t offset, uint32_t len)
{
  uint8_t addr[5];

  if (len > flashTotalSize || len+offset > flashTotalSize) {
    return 0;
  }

  addr[0] = FLASH_CMD_FAST_READ;
  addr[1] = ((offset >> 16) & 0xff);
  addr[2] = ((offset >> 8)  & 0xff);
  addr[3] = ((offset >> 0)  & 0xff);
  addr[4] = 0;

  exitDeepPowerDown();

  FLASH_CS_LOW;
  bsp_spiWrite(addr, 5);
  bsp_spiRead(buf, len );
  FLASH_CS_HIGH;


  return len;
}

/******************************************************************************
 *
 * Description:
 *    Write data to flash. Please note that the section where data is written
 *    must first have been erased.
 *
 * Params:
 *   [in] buf - data to write to flash
 *   [in] offset - offset into the flash
 *   [in] len - number of bytes to write
 *
 * Returns:
 *   number of written bytes
 *
 *****************************************************************************/
uint32_t flash_write(uint8_t* buf, uint32_t offset, uint32_t len)
{
  uint16_t wLen;
  uint32_t written = 0;
  uint8_t addr[5];

  if (len > flashTotalSize || len+offset > flashTotalSize) {
    return 0;
  }


  /* write up to first page boundry */
  wLen = (((( (offset%pageSize) / pageSize) + 1) * pageSize)
      - (offset%pageSize));
  wLen = MIN(wLen, len);

  while (len) {

    writeEnable();

    /* write address */
    addr[0] = FLASH_CMD_PP;
    addr[1] = ((offset >> 16) & 0xff);
    addr[2] = ((offset >> 8)  & 0xff);
    addr[3] = ((offset >> 0)  & 0xff);
    addr[4] = 0;


    FLASH_CS_LOW;
    bsp_spiWrite(addr, 4);
    bsp_spiWrite(&buf[written], wLen);
    FLASH_CS_HIGH;

    len     -= wLen;
    written += wLen;
    offset  += wLen;
    wLen = MIN(pageSize, len);

    pollIsBusy();
  }

  return written;
}

/******************************************************************************
 *
 * Description:
 *    Erase a sector
 *
 * Params:
 *   [in] offset - sector address to erase. Example: if the sector size
 *                 is 4K an offset between 0-4095 will erase the first sector.
 *                 An offset between 4096-8191 will erase the second sector,...
 *
 * Returns:
 *   1 if successful; otherwise 0
 *
 *****************************************************************************/
uint32_t flash_erase_sector(uint32_t offset)
{
  uint8_t addr[4];

  addr[0] = FLASH_CMD_SE;
  addr[1] = ((offset >> 16) & 0xff);
  addr[2] = ((offset >> 8)  & 0xff);
  addr[3] = ((offset >> 0)  & 0xff);

  if (offset > flashTotalSize) {
    return 0;
  }

  writeEnable();


  FLASH_CS_LOW;
  bsp_spiWrite(addr, 4);
  FLASH_CS_HIGH;


  pollIsBusy();

  return 1;
}

/******************************************************************************
 *
 * Description:
 *    Get flash page size
 *
 * Returns:
 *   The flash page size in bytes
 *
 *****************************************************************************/
uint32_t flash_getPageSize(void)
{
  return pageSize;
}

/******************************************************************************
 *
 * Description:
 *    Get flash sector size (one sector contains several pages)
 *
 * Returns:
 *   The flash sector size in bytes
 *
 *****************************************************************************/
uint32_t flash_getSectorSize(void)
{
  return sectorSize;
}


/******************************************************************************
 *
 * Description:
 *    Get total flash size
 *
 * Returns:
 *   The flash size in bytes
 *
 *****************************************************************************/
uint32_t flash_getTotalSize(void)
{
  return flashTotalSize;
}




