/**********************************************************************
 *
 * Copyright(C) 2011, Embedded Artists AB
 * All rights reserved.
 *
 ***********************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/



/******************************************************************************
 * Includes
 *****************************************************************************/

#include <string.h>
#include "image_fs.h"
#include "flash.h"

#include "images/text_image_2_7.xbm"
#include "images/ea.xbm"



/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/

typedef struct _image_fs_info
{
  uint32_t magic;
  uint32_t numImages;
  int32_t lastImage;
} __attribute__((packed)) imageFsInfo_t;

typedef struct _image_header
{
  uint32_t magic;
  uint32_t num;
} __attribute__((packed)) imageFsHeader_t;

#define IMAGE_FS_IMG_SIZE   (5808) // 264*176 / 8
#define IMAGE_FS_INFO_MAGIC (0xEA027AAE)
#define IMAGE_FS_HDR_MAGIC  (0xEE0102AA)


/******************************************************************************
 * External global variables
 *****************************************************************************/

/******************************************************************************
 * Local variables
 *****************************************************************************/

static uint32_t numImages = 0;
static uint8_t* readOnlyFs[2];
static uint32_t flashIsAvailable = FALSE;
static uint32_t usingReadOnlyFs = FALSE;


/******************************************************************************
 * Local Functions
 *****************************************************************************/

static uint32_t getSectorForImageOffset(uint32_t imgNum)
{
  int numSectors = 0;
  numSectors += ((IMAGE_FS_IMG_SIZE + flash_getPageSize()
      + flash_getSectorSize()-1) / flash_getSectorSize());
  return (imgNum+1) * numSectors * flash_getSectorSize();
}

//#define PROGRAM_IMAGES_IN_FLASH

#ifdef PROGRAM_IMAGES_IN_FLASH

#include "images\aphrodite_2_7.xbm"
#include "images\cat_2_7.xbm"
#include "images\saturn_2_7.xbm"
#include "images\text_hello_2_7.xbm"
#include "images\venus_2_7.xbm"

static uint32_t image_fs_write(uint32_t imgNum, uint8_t* buffer)
{
  uint32_t offset;
  imageFsHeader_t hdr;
  imageFsInfo_t fsInfo;
  uint32_t result = FALSE;
  uint32_t nImages = 0;
  int32_t lastImage = -1;
  int i = 0;
  int numSectors = 0;

  if (flashIsAvailable) {

    do {
      hdr.magic = IMAGE_FS_HDR_MAGIC;
      hdr.num   = imgNum;
      offset = getSectorForImageOffset(imgNum);

      // we first must erase the sectors that will be written
      numSectors = (IMAGE_FS_IMG_SIZE+flash_getPageSize()
          +flash_getSectorSize()-1)/flash_getSectorSize();
      for (i = 0; i < numSectors; i++) {
        flash_erase_sector(offset+i*flash_getSectorSize());
      }

      // write the image
      result = flash_write(buffer, offset+flash_getPageSize(),
          IMAGE_FS_IMG_SIZE);
      if (!result) break;

      // write the image header
      result = flash_write((uint8_t*)&hdr, offset, sizeof(imageFsHeader_t));
      if (!result) break;

      // update fs info section
      flash_read((uint8_t*)&fsInfo, 0, sizeof(imageFsInfo_t));
      if (fsInfo.magic == IMAGE_FS_INFO_MAGIC) {
        nImages = fsInfo.numImages;
        lastImage = fsInfo.lastImage;
      }

      if (imgNum >= nImages) {
        nImages = imgNum+1;
      }

      flash_erase_sector(0);
      fsInfo.magic = IMAGE_FS_INFO_MAGIC;
      fsInfo.numImages = nImages;
      fsInfo.lastImage = lastImage;
      flash_write((uint8_t*)&fsInfo, 0, sizeof(imageFsInfo_t));

    } while(FALSE);

  }

  return result;
}


static void storeImages()
{
  image_fs_write(0, (uint8_t*)ea_bits);
  image_fs_write(1, (uint8_t*)text_image_2_7_bits);
  image_fs_write(2, (uint8_t*)aphrodite_2_7_bits);
  image_fs_write(3, (uint8_t*)cat_2_7_bits);
  image_fs_write(4, (uint8_t*)saturn_2_7_bits);
  image_fs_write(5, (uint8_t*)text_hello_2_7_bits);
  image_fs_write(6, (uint8_t*)venus_2_7_bits);
}

#endif


/******************************************************************************
 * Public Functions
 *****************************************************************************/

/******************************************************************************
 *
 * Description:
 *    Initialize the Image file system
 *
 *****************************************************************************/
uint32_t image_fs_init (void)
{
  imageFsInfo_t fsInfo;


  if (flash_init()) {
    // a flash is available on Rev B on the e-paper module
    flashIsAvailable = TRUE;

#ifdef PROGRAM_IMAGES_IN_FLASH
    storeImages();
#endif

    numImages = 0;
    flash_read((uint8_t*)&fsInfo, 0, sizeof(imageFsInfo_t));
    if (fsInfo.magic == IMAGE_FS_INFO_MAGIC) {
      numImages = fsInfo.numImages;
    }
  } else {
	return FALSE;
  }


  if (!flashIsAvailable || numImages == 0) {
    numImages = 2;
    readOnlyFs[0] = (uint8_t*)&ea_bits[0];
    readOnlyFs[1] = (uint8_t*)&text_image_2_7_bits[0];
    usingReadOnlyFs = TRUE;
  }

  return TRUE;
}

/******************************************************************************
 *
 * Description:
 *    Returns number of images available in the file system
 *
 *****************************************************************************/
uint32_t image_fs_numImages()
{
  return numImages;
}

/******************************************************************************
 *
 * Description:
 *    Read specified image to given buffer
 *
 * Params:
 *   [in] imgNum - image to read
 *   [in] buffer - image will be read to this buffer
 *
 * Returns:
 *   number of read bytes or 0 if the request failed
 *
 *****************************************************************************/
uint32_t image_fs_read(uint32_t imgNum, uint8_t* buffer)
{
  uint32_t offset = 0;
  imageFsHeader_t hdr;
  uint32_t result = 0;

  if (usingReadOnlyFs) {
    memcpy(buffer, readOnlyFs[(imgNum%numImages)], IMAGE_FS_IMG_SIZE);
    result = IMAGE_FS_IMG_SIZE;
  } else {
    do {
      offset = getSectorForImageOffset(imgNum);
      result = flash_read((uint8_t*)&hdr, offset, sizeof(imageFsHeader_t));
      if (!result) break;
      if (hdr.magic != IMAGE_FS_HDR_MAGIC) break;

      result = flash_read(buffer, offset+flash_getPageSize(), IMAGE_FS_IMG_SIZE);
    } while (0);
  }

  return result;
}

/******************************************************************************
 *
 * Description:
 *    Store the image numbwe of the last shown image
 *
 * Params:
 *   [in] imgNum - image number
 *
 * Returns:
 *   non-zero if the request succeeded; otherwise 0
 *
 *****************************************************************************/
uint32_t image_fs_storeLastShown(uint32_t imgNum)
{
  imageFsInfo_t fsInfo;
  uint32_t nImages = 0;
  uint32_t result = 0;

  if (flashIsAvailable) {

    flash_read((uint8_t*)&fsInfo, 0, sizeof(imageFsInfo_t));
    if (fsInfo.magic == IMAGE_FS_INFO_MAGIC) {
      nImages = fsInfo.numImages;
    }

    // only update if there are images stored in the file system
    if (nImages > imgNum) {
      flash_erase_sector(0);
      fsInfo.magic = IMAGE_FS_INFO_MAGIC;
      fsInfo.numImages = nImages;
      fsInfo.lastImage = imgNum;
      result = flash_write((uint8_t*)&fsInfo, 0, sizeof(imageFsInfo_t));

    }
  }

  return result;
}

/******************************************************************************
 *
 * Description:
 *    Return the image number of the last shown image. -1 will be returned
 *    if there isn't any image number stored or if the request failed.
 *
 *****************************************************************************/
int32_t image_fs_lastShownImage()
{
  int32_t last = -1;
  imageFsInfo_t fsInfo;

  if (!usingReadOnlyFs) {

    flash_read((uint8_t*)&fsInfo, 0, sizeof(imageFsInfo_t));
    if (fsInfo.magic == IMAGE_FS_INFO_MAGIC) {
      last = fsInfo.lastImage;
    }

  }


  return last;
}

/******************************************************************************
 *
 * Description:
 *    Returns 1 if a flash (persistent storage) is available; otherwise 0.
 *
 *****************************************************************************/
uint32_t image_fs_isFlashAvailable()
{
  return flashIsAvailable;
}






