/*****************************************************************************
 *
 *   Copyright(C) 2013, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "lpc_types.h"

#include "bsp.h"
#include "epd.h"
#include "image_fs.h"
#include <string.h>


/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

#define pixel_width  264  // must be a multiple of 8
#define pixel_height 176

/******************************************************************************
 * Local Variables
 *****************************************************************************/

static uint8_t old_image[pixel_width * pixel_height / 8];
static uint8_t new_image[pixel_width * pixel_height / 8];

/******************************************************************************
 * Local Functions
 *****************************************************************************/
#if 0
// set a single pixel in new_image
static void drawPixel(int x, int y, unsigned int colour) {
  int bit = x & 0x07;
  int byte = x / 8 + y * (pixel_width / 8);
  int mask = 0x01 << bit;
  if (BLACK == colour) {
    new_image[byte] |= mask;
  } else {
    new_image[byte] &= ~mask;
  }
}
#endif


#if 0
static uint8_t reverse_bits(uint8_t b)
{
  int i = 0;
  uint8_t result = 0;

  for (i = 0; i < 8; i++) {
    if (b & (0x01 << i)) {
      result |= (0x80 >> i);
    }
  }

  return result;

}

static void memcpy_byte_flip(uint8_t* to, uint8_t* from, int numBytes)
{
  int i = 0;
  for (i = 0; i < numBytes; i++) {
    to[i] = reverse_bits(from[i]);
  }
}
#endif

typedef enum {
	TERR_OK,
	TERR_EPD_INIT,
	TERR_INVALID_FLASH,
	TERR_TEMPREADING,
} test_errors_t;

static void showError(test_errors_t terr) {
	bsp_setRgbLed(LED6_RED, 0);
	bsp_setRgbLed(LED6_GREEN, 0);
	bsp_setRgbLed(LED6_BLUE, 0);
	bsp_setRgbLed(LED7_RED, 0);
	bsp_setRgbLed(LED7_GREEN, 0);
	bsp_setRgbLed(LED7_BLUE, 0);

	switch (terr) {
	case TERR_OK:
		// Continuous GREEN light for success
		bsp_setRgbLed(LED6_GREEN, 1);
		bsp_setRgbLed(LED7_GREEN, 1);
		while(1) {
		}

	case TERR_EPD_INIT:
		// Continuous RED on LED6 and blinking RED on LED7
		bsp_setRgbLed(LED6_RED, 1);
		while(1) {
			bsp_setRgbLed(LED7_RED, 1);
		    bsp_delayMs(200);
			bsp_setRgbLed(LED7_RED, 0);
		    bsp_delayMs(200);
		}

	case TERR_INVALID_FLASH:
		// Continuous GREEN on LED6 and blinking RED on LED7
		bsp_setRgbLed(LED6_GREEN, 1);
		while(1) {
			bsp_setRgbLed(LED7_RED, 1);
		    bsp_delayMs(200);
			bsp_setRgbLed(LED7_RED, 0);
		    bsp_delayMs(200);
		}

	case TERR_TEMPREADING:
		// Continuous BLUE on LED6 and blinking RED on LED7
		bsp_setRgbLed(LED6_BLUE, 1);
		while(1) {
			bsp_setRgbLed(LED7_RED, 1);
		    bsp_delayMs(200);
			bsp_setRgbLed(LED7_RED, 0);
		    bsp_delayMs(200);
		}
	}
}

static int16_t getAndCheckTemp() {
    bsp_setRgbLed(LED6_BLUE, 1);
	int16_t temp = bsp_getTemp();
    bsp_setRgbLed(LED6_BLUE, 0);
    if (temp < 10 || temp > 50) {
    	showError(TERR_TEMPREADING);
    }
    return temp;
}

/******************************************************************************
 * Main method
 *****************************************************************************/
int main(void)
{
  int32_t imageCnt;
  int numImages = 0;
  int done = 0;

  bsp_init();
  bsp_startTimer(NULL);

  EPD_init(EPD_2_7);

  if (!image_fs_init()) {
	showError(TERR_INVALID_FLASH);
  }
  numImages = image_fs_numImages();

  bsp_setRgbLed(LED6_RED, 1);
  if (image_fs_isFlashAvailable()) {
    // turn off a LED on the board to indicate that a flash is
    // available on the e-paper module
    bsp_setRgbLed(LED6_RED, 0);
  }



  // clear buffers to white
  memset(old_image, 0, sizeof(old_image));
  memset(new_image, 0, sizeof(new_image));

  imageCnt = image_fs_lastShownImage();
  if (imageCnt == -1) {
    // if we failed to load last shown image clear the screen instead
	bsp_setRgbLed(LED6_GREEN, 1);
    EPD_begin();
    bsp_setRgbLed(LED6_GREEN, 0);
    EPD_setFactor(getAndCheckTemp());
    EPD_clear();
    EPD_end();

    imageCnt = 0;
  } else {

    // load the last shown image
    image_fs_read(imageCnt, old_image);
  }


  while(1)
  {
    if (imageCnt >= numImages) {
      imageCnt = 0;
      done = 1;
    }

    image_fs_read(imageCnt, new_image);

    // display new image
    EPD_begin();
    EPD_setFactor(getAndCheckTemp());
    EPD_image_sram(old_image, new_image);
    EPD_end();

    // copy new image to previous image
    memcpy(old_image, new_image, sizeof(old_image));

    image_fs_storeLastShown(imageCnt);
    imageCnt++;

    if (done) {
      // have shown all images without problem and have returned
      // to the logo again
      showError(TERR_OK);
    }

    //wait until key is released
    while (KEY_PRESS == bsp_getKey())
      ;
    bsp_delayMs(2000);
    //wait until key is pressed
    while (KEY_NO_PRESS == bsp_getKey())
      ;
  }


  return 1;
}


